/*
 * GenerateImages.java
 *
 * Created on February 12, 2006, 11:47 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.color.*;
import java.io.*;
import javax.imageio.*;
import javax.imageio.stream.*;

/**
 *
 * @author ek
 */
public class GenerateImages
{
    
    public static int SIZE = 128;
    
    // Create a grayscale color model.
    
    static ColorSpace cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
    static int cbits[] = new int[] {4};
    static ColorModel colormodel = new ComponentColorModel(cs, cbits, false, false,
        Transparency.OPAQUE,
        DataBuffer.TYPE_BYTE);
    
    
    static class FontPanel extends JPanel
    {
        
        Font thisFont;
        BufferedImage image;
        
        public FontPanel()
        {
            thisFont = new Font("Sazanami Mincho", Font.PLAIN, SIZE);
            setBackground(Color.WHITE);
            
            image = new BufferedImage(
                colormodel,
                colormodel.createCompatibleWritableRaster(SIZE, SIZE),
                false,
                new Hashtable());
            Graphics2D g = image.createGraphics();
            g.setBackground(Color.WHITE);
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, SIZE, SIZE);
            g.setColor(Color.BLACK);
            g.setFont(thisFont);
            g.drawString("\u76ee", 0, 54*SIZE/64);
            g.dispose();
        }
        
        public void paintComponent(Graphics g)
        {
            super.paintComponent( g );
            Graphics2D g2 = (Graphics2D) g;
            int w = getWidth();
            int h = getHeight();
            
            /*
            g2.setColor(Color.BLACK);
            g2.setFont(thisFont);
             
            String change = "\u76ee";
            FontMetrics metrics = g2.getFontMetrics();
            int width = metrics.stringWidth( change );
            int height = metrics.getHeight();
            System.out.println("String size: " + width + "x" + height);
            g2.drawString( change, 10, 100 );
             */
            
            g2.drawImage(image, 0, 0, null);
        }
    }
    
    
    public static void writePDB(byte[][] recordData) throws IOException
    {
        String db_name = "SamuraiKanjiData";
        DataOutputStream out = new DataOutputStream(
            new FileOutputStream(db_name + ".pdb"));
        byte[] b_db_name = db_name.getBytes();
        
        // 36 + 9*4 = 72 bytes of header
        
        out.write(b_db_name);
        for (int i=b_db_name.length; i<32; i++)
            out.write(0);
        
        out.writeShort(0x0008); // attributes: hotsync with desktop
        out.writeShort(0); // version
        out.writeInt(0);   // creation date
        out.writeInt(0);   // modification date
        out.writeInt(0);   // backup date
        out.writeInt(0);   // modification number
        out.writeInt(0);   // appInfoID offset
        out.writeInt(0);   // sortInfoID offset
        out.write("data".getBytes());   // type = data
        out.write("CHFY".getBytes());   // creator = CHFY
        out.writeInt(0);   // unique ID seed
        out.writeInt(0);   // next record list
        out.writeShort(recordData.length); // number of records
        
        // Total of 2 byte gap, plus 78 byte header, plus
        // 1 record headers times 8 bytes per record header
        
        int recordOffset = 2 + 78 + recordData.length * 8;
        
        for (int i=0; i<recordData.length; i++)
        {
            out.writeInt(recordOffset);
            out.writeInt(i); // attribute:byte and 3 bytes of unique record ID
            recordOffset += recordData[i].length;
        }
        
        out.writeShort(0); // traditional 2-byte gap between headers and records
        
        for (int i=0; i<recordData.length; i++)
            out.write(recordData[i]);
        
        out.flush();
        System.out.println("Wrote " + out.size() + " bytes to " + db_name + ".pdb");
        out.close();
    }
    
    static void writeImage(Raster img) throws Exception
    {
        ImageWriter writer = null;
        Iterator<ImageWriter> iter = ImageIO.getImageWritersByFormatName("png");
        if (iter.hasNext())
            writer = iter.next();
        
        javax.imageio.plugins.bmp.BMPImageWriteParam param = 
            (javax.imageio.plugins.bmp.BMPImageWriteParam)writer.getDefaultWriteParam();
        
        param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
        param.setCompressionType("BI_RLE8");

        ImageOutputStream ios = ImageIO.createImageOutputStream(new File ("tmp.bmp"));
            
        writer.setOutput(ios);
    
            // Set the compression quality
            // Write the image
        writer.write(null, new IIOImage(img, null, null), param);
    
            // Cleanup
            ios.flush();
            writer.dispose();
            ios.close();
         
    }
    
    static String[] readStrings(String filename) throws IOException
    {
        BufferedReader kin = new BufferedReader(
            new InputStreamReader(new FileInputStream(filename), "UTF-16"));
        ArrayList<String> list = new ArrayList<String>(200);
        String line;
        StringBuffer collect = new StringBuffer();
        
        while ( (line=kin.readLine()) != null )
        {
            line = line.trim();
            if (line.length() == 0) // skip blank lines
                continue;
            if (line.charAt(0) == '#') // skip lines beginning with #
            {
                if (line.startsWith("# Lesson")) // lesson marker
                {
                    if (collect.length() != 0)
                        list.add(collect.toString());
                    collect = new StringBuffer();
                }
                continue;
            }
            collect.append(line);
        }
        if (collect.length() != 0)
            list.add(collect.toString());
        
        System.out.println("Kanji file has " + list.size() + " lessons.");
        return list.toArray(new String[list.size()]);
    }
    
    static String[][] readStringArrays(String filename) throws IOException
    {
        BufferedReader kin = new BufferedReader(new FileReader(filename));
        ArrayList<String[]> list = new ArrayList<String[]>(200);
        ArrayList<String> collect = new ArrayList<String>(200);
        String line;
        
        while ( (line=kin.readLine()) != null )
        {
            line = line.trim();
            if (line.length() == 0) // skip blank lines
                continue;
            if (line.charAt(0) == '#') // skip lines beginning with #
            {
                if (line.startsWith("# Lesson")) // lesson marker
                {
                    if (collect.size() != 0)
                        list.add(collect.toArray(new String[collect.size()]));
                    collect = new ArrayList<String>(200);
                }
                continue;
            }
            collect.add(line);
        }
        if (collect.size() != 0)
            list.add(collect.toArray(new String[collect.size()]));
        
        System.out.println("Meanings file has " + list.size() + " lessons.");
        return list.toArray(new String[list.size()][]);
    }
    
    public static void main(String[] args) throws Exception
    {
        String[] lessons = readStrings("kanji.txt");
        String[][] meanings = readStringArrays("meaning.txt");
        
        // validate the files
        
        if (lessons.length != meanings.length)
        {
            System.out.println("Error: kanji.txt has " + lessons.length + 
                " lessons while meaning.txt has " + meanings.length +
                " lessons.");
            System.exit(1);
        }
        
        int totalKanji = 0;
        
        for (int i=0; i<lessons.length; i++)
        {
            if (lessons[i].length() != meanings[i].length)
            {
                System.out.println("Error: kanji.txt lesson " + (i+1) +
                    " has " + lessons[i].length() + " kanji " +
                    " while meaning.txt has " + meanings[i].length +
                    " meanings.");
                System.exit(1);
            }
            
            System.out.println("Lesson " + (i+1) + " starts with kanji " + 
                (totalKanji+1));
            totalKanji += lessons[i].length();
        }
        
        GraphicsEnvironment gEnv =
            GraphicsEnvironment.getLocalGraphicsEnvironment();
        String envfonts[] = gEnv.getAvailableFontFamilyNames();
        Font thisFont = new Font("Sazanami Mincho", Font.PLAIN, SIZE);
        
        // For some reason we need this, otherwise the characters don't
        // get anti-aliased.
        
        JFrame frame = new JFrame("Text Display");
        FontPanel panel = new FontPanel();
        frame.getContentPane().add(panel);
        frame.setSize(160, 160);
        // frame.setVisible(true);
        
        
        BufferedImage image = new BufferedImage(
            colormodel,
            colormodel.createCompatibleWritableRaster(SIZE, SIZE),
            false,
            new Hashtable());
        
        long pngsize = 0;
        totalKanji = 0;
        
        Vector<ByteArrayOutputStream> streams = new Vector<ByteArrayOutputStream>();
        
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(out);
        
        streams.add(out);
        
        for (int LESSON = 1; LESSON<=lessons.length; LESSON++)
        {
            String lesson = lessons[LESSON-1];
            
            for (int kanji=0; kanji<lesson.length(); kanji++)
            {
                // BufferedImage image = new BufferedImage(100, 100, BufferedImage.TYPE_BYTE_GRAY);
                Graphics2D g = image.createGraphics();
                
                // flip y axis for BMP compatibility
                
                AffineTransform tx = AffineTransform.getScaleInstance(1, -1);
                tx.translate(0, -image.getHeight(null));
                g.setTransform(tx); 
                
                g.setBackground(Color.WHITE);
                g.setColor(Color.WHITE);
                g.fillRect(0, 0, SIZE, SIZE);
                g.setColor(Color.BLACK);
                g.setFont(thisFont);
                g.drawString("" + lesson.charAt(kanji), 0, 54*SIZE/64);
                g.dispose();
                
                File file = new File("kanji.png");
                ImageIO.write(image, "png", file);
                long size = file.length();
                byte[] pngdata = new byte[(int)size];
                FileInputStream in = new FileInputStream(file);
                in.read(pngdata);
                in.close();
                
                dout.flush();
                if (out.size() + (int)size > 65000) // start a new record
                {
                    System.out.println("PDB record size: " + out.size());
                    out = new ByteArrayOutputStream();
                    dout = new DataOutputStream(out);
                    streams.add(out);
                }
                
                dout.writeByte(LESSON);
                dout.writeShort((short)size);
                
                byte[] b_meaning = meanings[LESSON-1][kanji].getBytes();
                dout.write(b_meaning);
                dout.writeByte(0); // terminate with NUL for C-style string
                
                dout.write(pngdata);
                pngsize += size;
                totalKanji++;
            }
        }
        
        dout.flush();
        System.out.println("PDB record size: " + out.size());
        System.out.println("PNG total size: " + pngsize);
        System.out.println("Total Kanji: " + totalKanji);
        
        byte[][] records = new byte[streams.size()][];
        for (int i=0; i<records.length; i++)
            records[i] = streams.get(i).toByteArray();
        
        writePDB(records);
        System.exit(0);
    }
}
