/*********************************************************************************
 *  Samurai Kanji, version 1.0                                                   *
 *  Copyright (C) 2006 Robert Baruch                                             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  Please see the software license located at /license.txt for more details.    *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Robert Baruch                                                             *
 *     76 Collins Lane                                                           *
 *     Rising Sun, MD  21911                                                     *
 *     USA                                                                       *
 *     E-mail: autophile@gmail.com                                               *
 *********************************************************************************/

import waba.io.*;
import waba.fx.*;

/**
 * A data stream that you can write 4096 pixels to, then retrieve
 * an Image representing the resulting 64 x 64 bitmap.
 *
 * @author Robert Baruch
 * @version 1.0
 */

public class Bmp extends DataStream
{
    final static int size = 128*128 + 1078;

    public Bmp()
    {
        super(new ByteArrayStream(new byte[size]));

        writeShort(0x424D);           // type = 'BM'
        writeIntLE(size);             // filesize
        writeIntLE(0);                // reserved 1 and 2
        writeIntLE(1078);             // image offset
        writeIntLE(40);               // header size from here
        writeIntLE(128);              // width
        writeIntLE(128);              // height
        writeShortLE(1);              // planes
        writeShortLE(8);              // bits per pixel
        writeIntLE(0);                // compression (0=uncompressed)
        writeIntLE(0);                // image size (0=uncompresed)
        writeIntLE(0);                // x pixels per meter (don't care)
        writeIntLE(0);                // y pixels per meter (don't care)
        writeIntLE(16);               // color map entries used
        writeIntLE(16);               // "important" color map entries

        int color = 0;
        for (int i=0; i<16; i++)    // color table
        {
            writeInt(color);
            color += 0x11111100;    // B, G, R, zero
        }
        for (int i=16; i<256; i++)
            writeInt(0);
    }

    public Image toImage()
    {
        return new Image( ((ByteArrayStream)getStream()).getBuffer() );
    }
}
